<?php

namespace Drupal\easy_entity_field\Plugin;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Provides the Easy entity field plugin manager.
 */
class EasyEntityFieldManager extends DefaultPluginManager {

  use StringTranslationTrait;

  /**
   * Constructs a new EasyEntityFieldManager object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/EasyEntityField', $namespaces, $module_handler, 'Drupal\easy_entity_field\Plugin\EasyEntityFieldPluginInterface', 'Drupal\easy_entity_field\Annotation\EasyEntityField');
    $this->alterInfo('easy_entity_field_info');
    $this->setCacheBackend($cache_backend, 'easy_entity_field_plugins');
  }

  /**
   * Return field types that can be added.
   *
   * @return array
   */
  public function getDefaultFields() {
    //Fields to be removed in the option list
    $field_types = [
      'uuid' => TRUE,
    ];

    //Fields to be removed from the hook
    $hook = 'easy_entity_field_remove_field_type';
    if ($this->moduleHandler->hasImplementations($hook)) {
      $field_types = NestedArray::mergeDeep($field_types, $this->moduleHandler->invokeAll($hook, [$field_types]));
    }

    /**
     * @var  \Drupal\Core\Field\FieldTypePluginManagerInterface $field_type_plugin_manager
     */
    $field_type_plugin_manager = \Drupal::service('plugin.manager.field.field_type');

    $field_type_options = [];
    foreach ($field_type_plugin_manager->getGroupedDefinitions($field_type_plugin_manager->getDefinitions()) as $category => $field_type_plugin) {
      foreach ($field_type_plugin as $name => $field_type) {
        // Remove fields from the option list
        if (!empty($field_types[$name]) && !$this->hasDefinition($name)) {
          continue;
        }

        $field_type_options[$category][$name] = $field_type['label'];
      }
    }

    return $field_type_options;
  }

}
